#ifndef KF_INTERFACE_H
#define KF_INTERFACE_H

//#ifndef EIGEN_DONT_ALIGN_STATICALLY
//#define EIGEN_DONT_ALIGN_STATICALLY
//#endif

#include <memory>
#include <mutex>
#include "iostream"
#include "Eigen/Core"
#include "Eigen/Dense"
#include "base_measure.h"
#include "rtk_measure.h"
#include "imu_measure.h"
#include "veh_measure.h"
#include "vloc_measure.h"
#include "common_define.h"
namespace Localization {
namespace Ego {

class KFInterface {
public:
    // init
    void InitQ();
    void InitP();
    void InitSensorParamVariance();
    // compute
    void PredictByimu(std::vector<std::shared_ptr<BaseMeasure>> &meaQue, State &tmpState);
    void Compute(std::shared_ptr<BaseMeasure> &ptrMea);
    void PredictToTime(const double &time, State &state);
    void PredictMean(const State &currState, double dt, State &nextState);
    void Propagate(const State &currState, double dt, State &nextState);
    bool CheckPredictJacobian(const State &state, const State &nextState, double dt,
         const Eigen::MatrixXd &mF);
    void Update(std::shared_ptr<BaseMeasure> &ptrMea, State &predictedstate, State &updatedState);
    void LimitState(std::shared_ptr<BaseMeasure> &ptrMea,State &pridictedState,State &updatedState);
  bool CrossCheck(std::shared_ptr<BaseMeasure> &ptrMea,State &updatedState);
    // interface
    void SetState(State &tempState);
    void ResetKF();
    void GetState(State &tempState);
    double GetStateTime();
    bool GetComputeFlag();
    bool GetInitFlag();
    void GetLocalizationResult(LocalizationResult &output, State &stateRes);
    void InitFilterState(std::shared_ptr<BaseMeasure> &mea);
private:
    // compute
    std::mutex mutexState_;
    bool computeFlag_ = true;
    bool initFlag_ = false;
    bool initPos_ = false;
    bool initAtt_ = false;
    bool initVelo_ = false;
    LocalizationResult KfOutput_;
    // filter
    State state_;
    Eigen::MatrixXd mQ_ = Eigen::Matrix3d::Zero();
    double accPropagateVar_ = 2.0 * 2.0;
    double avPropagateVar_ = 0.3 * 0.3;
    double imuAccRandomWalk_ = 0.0001 * 0.0001;
    double imuGyroRandomWalk_ = 0.00001 * 0.00001;
    double initRVar_ = 0.01 * 0.01;
    double initPVar_ = 0.5 * 0.5;
    double initVVar_ = 0.2 * 0.2;
    double initAccVar_ = 1.0 * 1.0; 
    double initAvVar_ = 0.01 * 0.01;
    double initImuAccBiasVar_ = 0.01 * 0.01; 
    double initImuGyroBiasVar_ = 0.001 * 0.001;
    double initWheelScaleVar_ = 0.05 * 0.05;
    double initImuInstallAngle_ = 0.01 * 0.01;
};

} // namespace Ego
} // namespace Localization
#endif
